#!/usr/bin/perl -w

# agglomerate.pl v0.2

# Program to take the training and test files output by
# sensevalOne2Two.pl and convert them into one big training file and
# one big test file

# Copyright (C) 2001-2002
# Ted Pedersen, University of Minnesota, Duluth
# tpederse@d.umn.edu
# Satanjeev Banerjee, University of Minnesota, Duluth
# bane0025@d.umn.edu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#-----------------------------------------------------------------------------
#                              Start of program
#-----------------------------------------------------------------------------

# we have to use commandline options, so use the necessary package!
use Getopt::Long;

# first check if no commandline options have been provided... in which case
# print out the usage notes!
if ($#ARGV == -1)
{
    &minimalUsageNotes();
    exit;
}

# now get the options!
GetOptions("version", "help", "train=s", "test=s", "remove_unmatched", "dir=s");

# if help has been requested, print out help!
if (defined $opt_help)
{
    $opt_help = 1;
    &showHelp();
    exit;
}

# if version has been requested, show version!
if (defined $opt_version)
{
    $opt_version = 1;
    &showVersion();
    exit;
}

# check if both --train and --test switches have been used. If not, complain.
if (!(defined $opt_train) || !(defined $opt_test))
{
    print STDERR "Options --train and --test options are both compulsory options.\n";
    askHelp();
    exit;
}

# open the train and test output files
open (TRAIN, ">$opt_train") || die ("Couldnt open $opt_train");
open (TEST, ">$opt_test") || die ("Couldnt open $opt_test");

# now get the directory. if not given, assume current directory.
$dir = (defined $opt_dir)?($opt_dir):".";

# now to get hold of the *.train.xml and *.test.xml files in the current directory
opendir DIR, $dir || "Couldnt open directory $dir\n";
my @trainFiles = grep /\.train\.xml$/, readdir DIR;
rewinddir DIR;
my @testFiles = grep /\.test\.xml$/, readdir DIR;
closedir DIR;

# if we couldnt find even one file, complain and quit!
if ($#trainFiles < 0 || $#testFiles < 0)
{
    print STDERR "Not enough files to continue!\n";
    askHelp();
    exit(-1);
}

# now, if remove_unmatched has been defined, pare the lists to keep only the necessary files.
if (defined $opt_remove_unmatched)
{
    $opt_remove_unmatched = 1;

    my @tempTrainFiles;
    my @tempTestFiles;
    my $trFile;

    foreach $trFile (@trainFiles)
    {
	$trFile =~ /([^.]*).train.xml/;
	my $trainStub = $1;
	
	my $teFile;
	foreach $teFile (@testFiles)
	{
	    $teFile =~ /([^.]*).test.xml/;
	    my $testStub = $1;
	    if ($trainStub eq $testStub)
	    {
		push @tempTrainFiles, $trFile;
		push @tempTestFiles, $teFile;
		last;
	    }
	}
    }

    @trainFiles = @tempTrainFiles;
    @testFiles = @tempTestFiles;
}

# now to do the actual putting into the train and test files.
print TRAIN "<corpus lang=\"en\">\n";

foreach $file (@trainFiles)
{
    open(SRC, "$dir/$file") || die("Couldnt open $dir/$file\n");

    my $copyFlag = 0;
    while (<SRC>)
    {
	if (/<lexelt/) { $copyFlag = 1; }
	if ($copyFlag)
	{
	    print TRAIN $_;
	}
	if (/<\/lexelt/) { $copyFlag = 0; }
    }
    close SRC;
}

print TRAIN "</corpus>\n";
close TRAIN;


# now to do the actual putting into the train and test files.
print TEST "<corpus lang=\"en\">\n";

foreach $file (@testFiles)
{
    open(SRC, "$dir/$file") || die("Couldnt open $dir/$file\n");

    my $copyFlag = 0;
    while (<SRC>)
    {
	if (/<lexelt/) { $copyFlag = 1; }
	if ($copyFlag)
	{
	    print TEST $_;
	}
	if (/<\/lexelt/) { $copyFlag = 0; }
    }
    close SRC;
}

print TEST "</corpus>\n";
close TEST;


# thats that!

# function to output a minimal usage note when the user has not provided any
# commandline options
sub minimalUsageNotes
{
    print "Usage: agglomerate.pl [OPTIONS] --train FILE_1 --test FILE_2\n";
    askHelp();
}

# function to output help messages for this program
sub showHelp
{
    print "Usage: agglomerate.pl [OPTIONS] --train FILE_1 --test FILE_2\n\n";
    print "Program to take the training and test files output by sensevalOne2Two.pl and\n";
    print "convert them into one training file FILE_1 and one test file FILE_2.\n\n";

    print "OPTIONS:\n\n";
    print "  --train FILE_1     Agglomerate files of the form *.train.xml into FILE_1\n\n";
    print "  --test FILE_2      Agglomerate files of the form *.test.xml into FILE_2\n\n";
    print "  --dir DIR          Specifies the directory wherein reside the test and train files\n";
    print "                     By default searches in the current directory.\n\n";
    print "  --remove_unmatched Remove all *.train.xml and *.test.xml files that do not have\n";
    print "                     a corresponding *.test.xml and *.train.xml file respectively\n";
    print "                     By default nothing is removed.\n\n";
    print "  --version          Prints the version number.\n\n";
    print "  --help             Prints this help message.\n\n";
}

# function to output the version number
sub showVersion
{
    print "agglomerate.pl -  Version 0.2\n";
    print "Copyright (C) 2001-2002, Ted Pedersen & Satanjeev Banerjee\n";
}

# function to output "ask for help" message when the user's goofed up!
sub askHelp
{
    print "Type agglomerate.pl --help for help.\n";
}
